-- File:    stdio_h_test.vhd
-- Version: 2.0	 (March 13, 2002)
-- Source:  http://bear.ces.cwru.edu/vhdl
-- Date:    March 13, 2002 (Copyright)
-- Author:  Francis G. Wolff   Email: fxw12@po.cwru.edu
-- Author:  Michael J. Knieser Email: mjknieser@knieser.com
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 1, or (at your option)
-- any later version: http://www.gnu.org/licenses/gpl.html
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
--
USE     STD.TEXTIO.ALL;
Library IEEE;
USE     IEEE.std_logic_1164.all;
LIBRARY C;
USE     C.stdio_h.all;
USE     C.strings_h.all;

entity stdio_h_test is
end;

architecture stdio_h_test_arch of stdio_h_test is
begin
  process
    variable v16: std_logic_vector(15 downto 0);
    variable i32:            integer:=0;
    variable fi, n:          integer;
    variable W, X:           line;
    variable s, s1, s2:      string(1 to 256);
    variable m1, m2, m3, m4: string(1 to 256);

    variable si,i: integer;
    variable bool: boolean;
    variable bit1: bit;
    variable b07:  bit_vector       (0 to 7):="00110101";
    variable b70:  bit_vector       (7 downto 0):="11001010";
    variable sl1:  std_logic:='1';
    variable v07:  std_logic_vector (0 to 7):="0LWXU1Z-"; --little endian
    variable v70:  std_logic_vector (7 downto 0):="-Z1UXWL0"; --big endian
    FILE     fout:         text;
    FILE     fin:          text;
    FILE     fout1, fout2: text;
    variable fbuf1, fbuf2: line;
  begin
    printf("abc "); printf("def\n"); --non line printf code

    sscanf("12AF", "%x", v16); printf("std_logic_vector(15 downto 0)=12AF=%s\n", v16);
    sscanf("12A",  "%x", v16); printf("std_logic_vector(15 downto 0)=012A=%s\n", v16);
    sscanf("27",   "%d", i32); printf("integer=27=%d\n", i32);

    strcpy(s, "  hello, world "); sscanf(s, "%s", s1); printf("%%s=%s\n", s1);
    strcpy(s, "  12ab 456   ");  printf("scanf=%s\n", s);
    strcpy(s, "  12ab 456   "); sscanf(s, "%d", n); printf("%%d=%d\n", n);
    strcpy(s, "  12ab 456   "); sscanf(s, "%x", s1); printf("%%s=%s\n", s1);
    strcpy(s, "  12ab 456   "); sscanf(s, "%x", n); printf("%%x=%x\n", n);
    strcpy(s, "  12ab 456   "); sscanf(s, "%o", n); printf("%%o=%o\n", n);
 
    -- all scanf vectors are little endian but will reverse for big endian vectors
    -- all printf vectors are printed as little endian
    strcpy(s, "  01001100 456   "); printf("scanf=%s\n", s);
    sscanf(s, "%s", bool); printf("bool %%s=%s\n", bool); printf("bool %%x=%x\n", bool);
    sscanf(s, "%s", bit1); printf("bit %%s=%s\n", bit1); printf("bit %%x=%x\n", bit1);
    strcpy(s, "  11001100 456   "); printf("scanf=%s\n", s);
    sscanf(s, "%s", bool); printf("bool %%s=%s\n", bool); printf("bool %%x=%x\n", bool);
    sscanf(s, "%s", bit1); printf("bit %%s=%s\n", bit1); printf("bit %%x=%x\n", bit1);
    sscanf(s, "%s", b07); printf("b07 %%s=%s\n", b07); printf("b07 %%x=%x\n", b07);
    sscanf(s, "%s", b70); printf("b70 %%s=%s\n", b70); printf("b70 %%x=%x\n", b70);
 
    strcpy(s, "  H10Xhlwu 456   "); printf("scanf=%s\n", s);
    sscanf(s, "%s", sl1); printf("std_logic %%s=%s\n", sl1); printf("sl1 %%x=%x\n", sl1);

    -- all scanf vectors are little endian but will reverse for big endian vectors
    sscanf(s, "%s", v07); printf("v07 %%s=%s\n", v07); printf("v07 %%x=%x\n", v07);
    sscanf(s, "%s", v70); printf("v70 %%s=%s\n", v70); printf("v70 %%x=%x\n", v70);

    printf("  hello,");
    printf("  world\n 123 9bc\n");
    putchar('%');
    puts("xyz++abc"); --LF added
    printf("***");
    putchar('%');
    puts("...def 555");

    file_open(fout, "xxx_out.txt", WRITE_MODE);
      --mixing fprintf, fputc, fputs with incomplete lines
      fprintf(fout, "  hello,");
      fprintf(fout, "  world\n 123 9bc\n");
      fputc('%', fout);
      fputs("xyz++abc" & LF, fout); --"\n" will not work here!
      fprintf(fout, "***");
      fputc('%', fout);
      fputs("...def 555" & LF, fout);
    fclose(fout);
 
    file_open(fout1, "xxx_fout1.txt", WRITE_MODE);
    file_open(fout2, "xxx_fout2.txt", WRITE_MODE);
      fprintf(fbuf1, fout1,  "  file1: ");
      fprintf(fbuf2, fout2, "  file2: ");
      fprintf(fbuf1, fout1,  "  Hello, World\n 123\n 9bc ");
      fprintf(fbuf2, fout2, "  Hallo, Welt\n 123\n 9bc ");
    fclose(fbuf1, fout1);
    fclose(fbuf2, fout2);

    file_open(fin, "xxx_out.txt", READ_MODE);
      --fscanf must use fbuf in order to stay with in a line
      fscanf(fin, "%3s", s); printf("s=hel==%s\n", s);

      fgetc(s(1), fin); 
      fgetc(s(2), fin); 
      fgetc(s(3), fin); 
      fgetc(s(4), fin);
      s(5):=NUL; --end of string

      printf("fgetc(1..5)=[lo, ]==[%s]\n", s);
      fscanf(fin, "%s", s); printf("s=world==%s\n", s);
      fscanf(fin, "%d", n); printf("s=123==%d\n", n);
      fscanf(fin, "%x", n); printf("s=9bc==%x\n", n);
    file_close(fin);

    printf("std_logic_vector=%s\n", std_logic_vector'("0X10010"));
    printf("std_logic=%s\n", std_logic'('1'));
    fprintf(output, "std_logic_vector=:%-10s:\n", pf(std_logic_vector'("0X10010")));
    fprintf(output, "std_logic       =:%-10s:\n", pf(std_logic'('1')));

    strcpy(s, "hello, world");
    printf("%%+ #-0.0s   :%+ #-0.0s:\n", s);
    printf("%%0.0s    :%0.0s:\n", s);
    printf("%%10s     :%10s:\n", s);
    printf("%%10.0s   :%10.0s:\n", s);
    printf("%%.10s    :%.10s:\n", s);
    printf("%%0.10s   :%0.10s:\n", s);
    printf("%%-10s    :%-10s:\n", s);
    printf("%%-10.0s  :%-10.0s:\n", s);
    printf("%%.15s    :%.15s:\n", s);
    printf("%%0.15s   :%0.15s:\n", s);
    printf("%%-15s    :%-15s:\n", s);
    printf("%%-15.0s  :%-15.0s:\n", s);
    printf("%%15.10s  :%15.10s:\n", s);
    printf("%%-15.10s :%-15.10s:\n", s);
    printf("%%-5.10s  :%-15.10s:\n", s);
    printf("true     =:%s:\n", true);
    printf("false    =:%s:P\n", false);
    printf("b07      =:%s:\n", b07);
    printf("b07      =:%s:\n", b07);
    printf("b70      =:%s:\n", b70);
    printf("v07      =:%s:\n", v07);
    printf("v70      =:%s:\n", v70);
    printf("std_logic=:%s:\n", sl1);
    printf("%%s   -15=:%s:\n", -15);
    printf("%%s    -1=:%s:\n", -1);
    printf("%%d    -1=:%d:\n", -1);
    printf("%%s    -7=:%s:\n", -7);
    printf("%%u    -7=:%u:\n", -7);
    printf("%%d    -7=:%d:\n", -7);
    printf("%%s    15=:%s:\n", 15);
    printf("%%d    15=:%d:\n", 15);
    printf("%%#3d    =:%#3d:\n",  1945);
    printf("%% 3d    =:% 3d:\n",  1945);
    printf("%% +3d   =:% +3d:\n", 1945);
    printf("%%+ 3d   =:%+ 3d:\n", 1945);
    printf("%%+3d    =:%+3d:\n",  1945);
    printf("%%3d     =:%3d:\n",   1945);
    printf("%%3d     =:%3d:\n",  -1945);
    printf("%%10x    =:%10x:\n",  1945);
    printf("%%10x    =:%10x:\n", -1945);
    printf("%%10x    =:%#10x:\n", 1945);
    printf("%%10x    =:%#10x:\n", -1945);
    printf("%%s -1945=:%s:\n",    -1945);
    printf("%%10d    =:%10d:\n",    1945);
    printf("%%10d    =:%10d:\n",   -1945);
    printf("%%010d   =:%010d:\n",   1945);
    printf("%%010d   =:%010d:\n",  -1945);
    printf("%% 010d  =:% 010d:\n",  1945);
    printf("%% 010d  =:% 010d:\n", -1945);
    printf("%%+010d  =:%+010d:\n",  1945);
    printf("%%+010d  =:%+010d:\n", -1945);
    printf("bit_vector       %%s    =:%s:\n",   pf(bit_vector'("10011001111"))); -- =1945
    printf("bit_vector       %%10s  =:%10s:\n", pf(bit_vector'("10011001111"))); -- =1945
    printf("bit_vector       %%d    =:%d:\n",   pf(bit_vector'("10011001111")));
    printf("bit_vector       %%10d  =:%10d:\n", pf(bit_vector'("10011001111")));
    printf("std_logic_vector %%d    =:%d:\n",   std_logic_vector'("10011001111"));
    printf("std_logic_vector %%10d  =:%10d:\n", std_logic_vector'("10011001111"));
    printf("std_logic_vector %%u    =:%u:\n",   std_logic_vector'("10011001111"));
    printf("std_logic_vector %%10u  =:%10u:\n", std_logic_vector'("10011001111"));
    printf("std_logic_vector %%x    =:%x:\n",   std_logic_vector'("10011001111"));
    printf("std_logic_vector %%#x   =:%x:\n",   std_logic_vector'("10011001111"));
    printf("std_logic_vector %%10x  =:%10x:\n", std_logic_vector'("10011001111"));
    printf("std_logic_vector %%o    =:%o:\n",   std_logic_vector'("10011001111"));
    printf("std_logic_vector %%#o   =:%o:\n",   std_logic_vector'("10011001111"));
    printf("std_logic_vector %%10o  =:%10o:\n", std_logic_vector'("10011001111"));


    -- special debugging stdio_h internal functions

    write(output, string'("helloX"));
    write(output, string'("helloY"));

    write(W, string'("hello"));
    printf("s=(%s)\n", w.all);
    printf("i=%d\n", w'length);

    deallocate(W);

    fi:=1; sbufprintf(false, fi, W, output, "hello1\nhello2\n", " ");
    printf("w'length==0==%d (%s)\n", w'length, w.all);

    deallocate(W);
    fi:=1; sbufprintf(true, fi, W, output, "hello3\nhello4\n", " ");
    printf("sprintf=(%s)\n", w.all);
    printf("w'length==14==%d\n", w'length);
    W:=null; deallocate(W);
    deallocate(W);
    deallocate(W);

    write(X, string'(" 123abch  456  "));
    fi:=1; sbufscanf(true, fi, X, output, "%s", W);
    printf("sscanf unused=(%s)\n", x.all);
    printf("sscanf matched=(%s)\n", w.all);
    printf("i=%d\n", w'length);
   
    deallocate(X); deallocate(W);
    write(X, string'(" 123abch  456  "));
    fi:=1; sbufscanf(true, fi, X, output, "%2s", W);
    printf("sscanf unused=(%s)\n", x.all);
    printf("sscanf matched=(%s)\n", w.all);
    printf("i=%d\n", w'length);

    deallocate(X); deallocate(W);
    write(X, string'(" 123abch  456   "));
    fi:=1; sbufscanf(true, fi, X, output, "%d", W);
    printf("sscanf unused=(%s)\n", x.all);
    printf("sscanf matched=(%s)\n", w.all);
    printf("i=%d\n", w'length);

    deallocate(X); deallocate(W);
    write(X, string'(" 123abch   456   "));
    fi:=1; sbufscanf(true, fi, X, output, "%x", W);
    printf("sscanf unused=(%s)\n", x.all);
    printf("sscanf matched=(%s)\n", w.all);
    printf("sscanf matched=(%x)\n", w.all);
    printf("i=%d\n", w'length);
    wait;
  end process;
end;

configuration stdio_h_test_cfg of stdio_h_test is
  for stdio_h_test_arch
  end for;
end;
